# Import python libs
import datetime
import dateutil.tz
import json
import socket
import re
import timeit
import requests

# Import third party libs
import boto3
import docker
import docker.errors
from botocore.exceptions import ClientError
from cachetools import cached, TTLCache

# Import metadataproxy libs
from metadataproxy import app
from metadataproxy import log

ROLES = {}
CONTAINER_MAPPING = {}
_docker_client = None
_iam_client = None
_sts_client = None

if app.config['ROLE_MAPPING_FILE']:
    with open(app.config.get('ROLE_MAPPING_FILE'), 'r') as f:
        ROLE_MAPPINGS = json.loads(f.read())
else:
    ROLE_MAPPINGS = {}

RE_IAM_ARN = re.compile(r"arn:aws:iam::(\d+):role/(.*)")


class BlockTimer(object):
    def __enter__(self):
        self.start_time = timeit.default_timer()
        return self

    def __exit__(self, *args):
        self.end_time = timeit.default_timer()
        self.exec_duration = self.end_time - self.start_time


class PrintingBlockTimer(BlockTimer):
    def __init__(self, prefix=''):
        self.prefix = prefix

    def __exit__(self, *args):
        super(PrintingBlockTimer, self).__exit__(*args)
        msg = "Execution took {0:f}s".format(self.exec_duration)
        if self.prefix:
            msg = self.prefix + ': ' + msg
        log.debug(msg)


def log_exec_time(method):
    def timed(*args, **kw):
        with PrintingBlockTimer(method.__name__):
            result = method(*args, **kw)
        return result
    return timed


def docker_client():
    global _docker_client
    if _docker_client is None:
        _docker_client = docker.Client(base_url=app.config['DOCKER_URL'])
    return _docker_client


def iam_client():
    global _iam_client
    if _iam_client is None:
        _iam_client = boto3.client('iam')
    return _iam_client


def sts_client():
    global _sts_client
    if _sts_client is None:
        _sts_client = boto3.client('sts')
    return _sts_client


@log_exec_time
def find_container(ip):
    pattern = re.compile(app.config['HOSTNAME_MATCH_REGEX'])
    client = docker_client()
    # Try looking at the container mapping cache first
    container_id = CONTAINER_MAPPING.get(ip)
    if container_id:
        log.info('Container id for IP {0} in cache'.format(ip))
        try:
            with PrintingBlockTimer('Container inspect'):
                container = client.inspect_container(container_id)
            # Only return a cached container if it is running.
            if container['State']['Running']:
                return container
            else:
                log.error('Container id {0} is no longer running'.format(ip))
                if ip in CONTAINER_MAPPING:
                    del CONTAINER_MAPPING[ip]
        except docker.errors.NotFound:
            msg = 'Container id {0} no longer mapped to {1}'
            log.error(msg.format(container_id, ip))
            if ip in CONTAINER_MAPPING:
                del CONTAINER_MAPPING[ip]

    _fqdn = None
    with PrintingBlockTimer('Reverse DNS'):
        if app.config['ROLE_REVERSE_LOOKUP']:
            try:
                _fqdn = socket.gethostbyaddr(ip)[0]
            except socket.error as e:
                log.error('gethostbyaddr failed: {0}'.format(e.args))
                pass

    with PrintingBlockTimer('Container fetch'):
        _ids = [c['Id'] for c in client.containers()]

    for _id in _ids:
        try:
            with PrintingBlockTimer('Container inspect'):
                c = client.inspect_container(_id)
        except docker.errors.NotFound:
            log.error('Container id {0} not found'.format(_id))
            continue
        # Try matching container to caller by IP address
        _ip = c['NetworkSettings']['IPAddress']
        if ip == _ip:
            msg = 'Container id {0} mapped to {1} by IP match'
            log.debug(msg.format(_id, ip))
            CONTAINER_MAPPING[ip] = _id
            return c
        # Try matching container to caller by sub network IP address
        _networks = c['NetworkSettings']['Networks']
        if _networks:
            for _network in _networks:
                if _networks[_network]['IPAddress'] == ip:
                    msg = 'Container id {0} mapped to {1} by sub-network IP match'
                    log.debug(msg.format(_id, ip))
                    CONTAINER_MAPPING[ip] = _id
                    return c
        # Not Found ? Let's see if we are running under rancher 1.2+,which uses a label to store the IP
        try:
            _labels = c.get('Config', {}).get('Labels', {})
        except (KeyError, ValueError):
            _labels = {}
        try:
            if _labels.get('io.rancher.container.ip'):
                _ip = _labels.get('io.rancher.container.ip').split("/")[0]
        except docker.errors.NotFound:
            log.error('Container: {0} Label container.ip not found'.format(_id))
        if ip == _ip:
            msg = 'Container id {0} mapped to {1} by Rancher IP match'
            log.debug(msg.format(_id, ip))
            CONTAINER_MAPPING[ip] = _id
            return c
        # Try matching container to caller by hostname match
        if app.config['ROLE_REVERSE_LOOKUP']:
            hostname = c['Config']['Hostname']
            domain = c['Config']['Domainname']
            fqdn = '{0}.{1}'.format(hostname, domain)
            # Default pattern matches _fqdn == fqdn
            _groups = re.match(pattern, _fqdn).groups()
            groups = re.match(pattern, fqdn).groups()
            if _groups and groups:
                if groups[0] == _groups[0]:
                    msg = 'Container id {0} mapped to {1} by FQDN match'
                    log.debug(msg.format(_id, ip))
                    CONTAINER_MAPPING[ip] = _id
                    return c
        # Try to find the container over the mesos state api and use the labels attached to it
        # as a replacement for docker env and labels
        if app.config['MESOS_STATE_LOOKUP']:
            mesos_container = find_mesos_container(ip)
            if mesos_container is not None:
                return mesos_container

    log.error('No container found for ip {0}'.format(ip))
    return None


@cached(cache=TTLCache(maxsize=512, ttl=60))
@log_exec_time
def find_mesos_container(ip):
    mesos_state_url = app.config['MESOS_STATE_URL']
    try:
        state = requests.get(mesos_state_url, timeout=app.config['MESOS_STATE_TIMEOUT']).json()
        for framework in state['frameworks']:
            for executor in framework['executors']:
                for task in executor['tasks']:
                    for status in task['statuses']:
                        if status['state'] == 'TASK_RUNNING':
                            for network in status['container_status']['network_infos']:
                                for ip_map in network['ip_addresses']:
                                    if ip_map['ip_address'] == ip:
                                        if 'labels' in task:
                                            env = []
                                            for label in task['labels']:
                                                key = label['key']
                                                val = label['value']
                                                env_var = '{0}={1}'.format(key, val)
                                                env.append(env_var)
                                            container = {'Config': {'Env': env, 'Labels': env}}
                                            return container

    except requests.exceptions.Timeout:
        log.error('Timeout when trying to call the mesos http api: {0}'.format(mesos_state_url))
    except requests.exceptions.RequestException:
        log.exception('Error while trying to call the mesos http api: {0}'.format(mesos_state_url))
    except KeyError:
        log.exception('Error while trying to lookup the required keys in the json object')
    return None


def split_envvar(envvar):
    """Splits str formatted as `key=val` into [key, val]

    if string is missing an `=val` it will return [key, None]
    """
    return (envvar.split('=', 1) + [None])[:2]


@log_exec_time
def get_role_params_from_ip(ip, requested_role=None):
    params = {'name': None, 'account_id': None, 'external_id': None, 'session_name': None}
    role_name = None
    if app.config['ROLE_MAPPING_FILE']:
        role = ROLE_MAPPINGS.get(ip, app.config['DEFAULT_ROLE'])
        if isinstance(role, dict):
            params.update(role)
        else:
            role_name = role
    else:
        container = find_container(ip)
        if container:
            env = container['Config']['Env'] or []
            # Look up IAM_ROLE and IAM_EXTERNAL_ID values from environment
            for e in env:
                key, val = split_envvar(e)
                if key == 'IAM_ROLE':
                    m = RE_IAM_ARN.match(val)
                    if m:
                        val = '{0}@{1}'.format(m.group(2), m.group(1))
                    role_name = val
                elif key == 'IAM_EXTERNAL_ID':
                    params['external_id'] = val
            if not role_name:
                msg = "Couldn't find IAM_ROLE variable. Returning DEFAULT_ROLE: {0}"
                log.debug(msg.format(app.config['DEFAULT_ROLE']))
                role_name = app.config['DEFAULT_ROLE']

            # Optionally, look up role session name from environment or labels
            if app.config['ROLE_SESSION_KEY']:
                skey = app.config['ROLE_SESSION_KEY']
                sval = None
                if skey.startswith('Env:'):
                    skey = skey[4:]
                    for e in env:
                        key, val = split_envvar(e)
                        if skey == key:
                            sval = val
                elif skey.startswith('Labels:'):
                    skey = skey[7:]
                    if container['Config']['Labels'] and skey in container['Config']['Labels']:
                        sval = container['Config']['Labels'][skey]
                if sval and len(sval) > 1:
                    # The docs on RoleSessionName are slightly contradictory, and state:
                    # > The regex used to validate this parameter is a string of characters consisting
                    # > of upper- and lower-case alphanumeric characters with no spaces. You can also
                    # > include underscores or any of the following characters: =,.@-
                    # > Type: String
                    # > Length Constraints: Minimum length of 2. Maximum length of 64.
                    # > Pattern: [\w+=,.@-]*
                    # We replace any invalid chars with underscore, and trim to 64.
                    params['session_name'] = re.sub(r'[^\w+=,.@-]', '_', sval)[:64]
    if role_name:
        role_parts = role_name.split('@')
        params['name'] = role_parts[0]
        if len(role_parts) > 1:
            params['account_id'] = role_parts[1]

    if requested_role and requested_role != params['name']:
        raise UnexpectedRoleError

    return params


@log_exec_time
def get_role_info_from_params(role_params):
    if not role_params['name']:
        return {}
    try:
        role = get_assumed_role(role_params)
    except GetRoleError:
        return {}
    time_format = "%Y-%m-%dT%H:%M:%SZ"
    expiration = role['Credentials']['Expiration']
    updated = expiration - datetime.timedelta(minutes=60)
    return {
        'Code': 'Success',
        'LastUpdated': updated.strftime(time_format),
        'InstanceProfileArn': role['AssumedRoleUser']['Arn'],
        'InstanceProfileId': role['AssumedRoleUser']['AssumedRoleId']
    }


def get_role_arn(role_params):
    if role_params['account_id']:
        # Try to map the name to an account ID. If it isn't found, assume an ID was passed
        # in and use it as-is.
        role_params['account_id'] = app.config['AWS_ACCOUNT_MAP'].get(
            role_params['account_id'],
            role_params['account_id']
        )
    else:
        if app.config['DEFAULT_ACCOUNT_ID']:
            role_params['account_id'] = app.config['DEFAULT_ACCOUNT_ID']
        # No default account id defined. Get the ARN by looking up the role
        # name. This is a backwards compat use-case for when we didn't require
        # the default account id.
        else:
            iam = iam_client()
            try:
                with PrintingBlockTimer('iam.get_role'):
                    if '/' in role_params['name']:
                        path, name = role_params['name'].rsplit('/', 1)
                        role = iam.get_role(Path=path + '/', RoleName=name)
                    else:
                        role = iam.get_role(RoleName=role_params['name'])
                    return role['Role']['Arn']
            except ClientError as e:
                response = e.response['ResponseMetadata']
                raise GetRoleError((response['HTTPStatusCode'], e.message))
    # Return a generated ARN
    return 'arn:aws:iam::{account_id}:role/{name}'.format(**role_params)


@log_exec_time
def get_assumed_role(role_params):
    arn = get_role_arn(role_params)
    if arn in ROLES:
        assumed_role = ROLES[arn]
        expiration = assumed_role['Credentials']['Expiration']
        now = datetime.datetime.now(dateutil.tz.tzutc())
        expire_check = now + datetime.timedelta(minutes=5)
        if expire_check < expiration:
            return assumed_role
    with PrintingBlockTimer('sts.assume_role'):
        sts = sts_client()
        session_name = role_params['session_name'] or 'devproxyauth'
        kwargs = {'RoleArn': arn, 'RoleSessionName': session_name}
        if role_params['external_id']:
            kwargs['ExternalId'] = role_params['external_id']
        assumed_role = sts.assume_role(**kwargs)
    ROLES[arn] = assumed_role
    return assumed_role


@log_exec_time
def get_assumed_role_credentials(role_params, api_version='latest'):
    assumed_role = get_assumed_role(role_params)
    time_format = "%Y-%m-%dT%H:%M:%SZ"
    credentials = assumed_role['Credentials']
    expiration = credentials['Expiration']
    updated = expiration - datetime.timedelta(minutes=60)
    return {
        'Code': 'Success',
        'LastUpdated': updated.strftime(time_format),
        'Type': 'AWS-HMAC',
        'AccessKeyId': credentials['AccessKeyId'],
        'SecretAccessKey': credentials['SecretAccessKey'],
        'Token': credentials['SessionToken'],
        'Expiration': expiration.strftime(time_format)
    }


class GetRoleError(Exception):
    pass


class UnexpectedRoleError(Exception):
    pass



import json
from os import getenv


def bool_env(var_name, default=False):
    """
    Get an environment variable coerced to a boolean value.
    Example:
        Bash:
            $ export SOME_VAL=True
        settings.py:
            SOME_VAL = bool_env('SOME_VAL', False)
    Arguments:
        var_name: The name of the environment variable.
        default: The default to use if `var_name` is not specified in the
                 environment.
    Returns: `var_name` or `default` coerced to a boolean using the following
        rules:
            "False", "false" or "" => False
            Any other non-empty string => True
    """
    test_val = getenv(var_name, default)
    # Explicitly check for 'False', 'false', and '0' since all non-empty
    # string are normally coerced to True.
    if test_val in ('False', 'false', '0'):
        return False
    return bool(test_val)


def float_env(var_name, default=0.0):
    """
    Get an environment variable coerced to a float value.
    This has the same arguments as bool_env. If a value cannot be coerced to a
    float, a ValueError will be raised.
    """
    return float(getenv(var_name, default))


def int_env(var_name, default=0):
    """
    Get an environment variable coerced to an integer value.
    This has the same arguments as bool_env. If a value cannot be coerced to an
    integer, a ValueError will be raised.
    """
    return int(getenv(var_name, default))


def str_env(var_name, default=''):
    """
    Get an environment variable as a string.
    This has the same arguments as bool_env.
    """
    return getenv(var_name, default)


PORT = int_env('PORT', 45001)
HOST = str_env('HOST', '0.0.0.0')
DEBUG = bool_env('DEBUG', False)

# Url of the docker daemon. The default is to access docker via its socket.
DOCKER_URL = str_env('DOCKER_URL', 'unix://var/run/docker.sock')
# URL of the metadata service. Default is the normal location of the
# metadata service in AWS.
METADATA_URL = str_env('METADATA_URL', 'http://169.254.169.254')
# Whether or not to mock all metadata endpoints. If True, mocked data will be
# returned to callers. If False, all endpoints except for IAM endpoints will be
# proxied through to the real metadata service.
MOCK_API = bool_env('MOCK_API', False)
# When mocking the API, use the following instance id in returned data.
MOCKED_INSTANCE_ID = str_env('MOCKED_INSTANCE_ID', 'mockedid')

# Role to use if IAM_ROLE is not set in a container's environment. If unset
# the container will get no IAM credentials.
DEFAULT_ROLE = str_env('DEFAULT_ROLE')
# The default account ID to assume roles in, if IAM_ROLE does not contain
# account information. If unset, metadataproxy will attempt to lookup role
# ARNs using IAM:GET_ROLE, if the IAM_ROLE name is not an ARN.
DEFAULT_ACCOUNT_ID = str_env('DEFAULT_ACCOUNT_ID')
# A mapping of account names to account IDs. This allows you to use
# user-friendly names in the IAM_ROLE environment variable; for instance:
#
#   AWS_ACCOUNT_MAP={'my-account-name':'12345'}
#
# A lookup of myrole@my-account-name would map to
#
#   role_name: myrole
#   account_id: 12345
AWS_ACCOUNT_MAP = json.loads(str_env('AWS_ACCOUNT_MAP', '{}'))

# A json file that has a dict mapping of IP addresses to role names. Can be
# used if docker networking has been disabled and you are managing IP
# addressing for containers through another process.
ROLE_MAPPING_FILE = str_env('ROLE_MAPPING_FILE')
# Do a reverse lookup of incoming IP addresses to match containers by hostname.
# Useful if you've disabled networking in docker, but set hostnames for
# containers in /etc/hosts or DNS.
ROLE_REVERSE_LOOKUP = bool_env('ROLE_REVERSE_LOOKUP', False)
# Limit reverse lookup container matching to hostnames that match the specified
# pattern.
HOSTNAME_MATCH_REGEX = str_env('HOSTNAME_MATCH_REGEX', '^.*$')
# Optional key in container labels or environment variables to use for role session name.
# Prefix with Labels: or Env: respectively to indicate where key should be found.
ROLE_SESSION_KEY = str_env('ROLE_SESSION_KEY')
# In case we also want to query the mesos state api
MESOS_STATE_LOOKUP = bool_env('MESOS_STATE_LOOKUP', False)
# URL of the mesos state endpoint to query
MESOS_STATE_URL = str_env('MESOS_STATE_URL', 'http://localhost:5051/state')
# Timeout to use when calling the mesos state endpoint
MESOS_STATE_TIMEOUT = int_env('MESOS_STATE_TIMEOUT', 2)



from flask import Response
from flask import request
from flask import stream_with_context
from flask import jsonify

import requests

from metadataproxy import app
from metadataproxy import log
from metadataproxy import roles


def _supports_iam(version):
    '''Check the meta-data version for IAM support

    API versions before 2012-01-12 don't support the iam/ subtree.
    This function works because:
    >>> '1.0' < '2007-01-19' < '2014-11-05' < 'latest'
    True
    '''
    return version >= '2012-01-12'


@app.route('/<api_version>/meta-data/iam/info', strict_slashes=False)
@app.route('/<api_version>/meta-data/iam/info/<path:junk>')
def iam_role_info(api_version, junk=None):
    if not _supports_iam(api_version):
        return passthrough(request.path)

    role_params_from_ip = roles.get_role_params_from_ip(request.remote_addr)
    if role_params_from_ip['name']:
        log.debug('Providing IAM role info for {0}'.format(role_params_from_ip['name']))
        return jsonify(roles.get_role_info_from_params(role_params_from_ip))
    else:
        log.error('Role name not found; returning 404.')
        return '', 404


@app.route('/<api_version>/meta-data/iam/security-credentials/')
def iam_role_name(api_version):
    if not _supports_iam(api_version):
        return passthrough(request.path)

    role_params_from_ip = roles.get_role_params_from_ip(request.remote_addr)
    if role_params_from_ip['name']:
        return role_params_from_ip['name']
    else:
        log.error('Role name not found; returning 404.')
        return '', 404


@app.route('/<api_version>/meta-data/iam/security-credentials/<path:requested_role>',
           strict_slashes=False)
def iam_sts_credentials(api_version, requested_role):
    if not _supports_iam(api_version):
        return passthrough(request.path)

    try:
        role_params = roles.get_role_params_from_ip(
            request.remote_addr,
            requested_role=requested_role.rstrip('/')
        )
    except roles.UnexpectedRoleError:
        msg = "Role name {0} doesn't match expected role for container"
        log.error(msg.format(requested_role))
        return '', 404

    log.debug('Providing assumed role credentials for {0}'.format(role_params['name']))
    assumed_role = roles.get_assumed_role_credentials(
        role_params=role_params,
        api_version=api_version
    )
    return jsonify(assumed_role)


@app.route('/<path:url>')
@app.route('/')
def passthrough(url=''):
    log.debug('Did not match credentials request url; passing through.')
    req = requests.get(
        '{0}/{1}'.format(app.config['METADATA_URL'], url),
        stream=True
    )
    return Response(
        stream_with_context(req.iter_content()),
        content_type=req.headers['content-type'],
        status=req.status_code
    )



import datetime
import dateutil

from flask import request
from flask import redirect
from flask import url_for
from flask import jsonify

from metadataproxy import app
from metadataproxy import log
from metadataproxy import roles


@app.route(
    '/<api_version>'
)
def root_noslash(api_version):
    return redirect(
        url_for('root_slash', api_version=api_version),
        code=301
    )


@app.route(
    '/<api_version>/'
)
def root_slash(api_version):
    return 'meta-data', 200


@app.route(
    '/<api_version>/meta-data'
)
def get_meta_data_noslash(api_version):
    return redirect(
        url_for('get_meta_data_slash', api_version=api_version),
        code=301
    )


@app.route('/<api_version>/meta-data/')
def get_meta_data_slash(api_version):
    meta_data_list = [
        'ami-id', 'ami-launch-index', 'ami-manifest-path',
        'block-device-mapping/', 'hostname', 'iam/', 'instance-action',
        'instance-id', 'instance-type', 'local-hostname', 'local-ipv4',
        'mac', 'metrics/', 'network/', 'placement/', 'profile',
        'public-hostname', 'public-ipv4', 'public-keys/', 'reservation-id',
        'security-groups', 'services/'
    ]
    return '\n'.join(meta_data_list), 200


@app.route('/<api_version>/meta-data/ami-id')
def get_ami_id(api_version):
    return 'ami-mockedami', 200


@app.route('/<api_version>/meta-data/ami-launch-index')
def get_ami_launch_index(api_version):
    return '0', 200


@app.route('/<api_version>/meta-data/ami-manifest-path')
def get_ami_manifest_path(api_version):
    return '(unknown)', 200


@app.route('/<api_version>/meta-data/block-device-mapping')
def get_block_device_mapping_noslash(api_version):
    return redirect(
        url_for('get_block_device_mapping_slash', api_version=api_version),
        code=301
    )


@app.route('/<api_version>/meta-data/block-device-mapping/')
def get_block_device_mapping_slash(api_version):
    return 'ami\nroot', 200


@app.route('/<api_version>/meta-data/block-device-mapping/ami')
def get_block_device_mapping_ami(api_version):
    return '/dev/sda1', 200


@app.route('/<api_version>/meta-data/block-device-mapping/root')
def get_block_device_mapping_root(api_version):
    return '/dev/sda1', 200


@app.route('/<api_version>/meta-data/hostname')
def get_hostname(api_version):
    return 'mocked.internal', 200


@app.route('/<api_version>/meta-data/iam')
def get_iam_noslash(api_version):
    return redirect(
        url_for('get_iam_slash', api_version=api_version),
        301
    )


@app.route('/<api_version>/meta-data/iam/')
def get_iam_slash(api_version):
    return 'info\nsecurity-credentials/', 200


@app.route('/<api_version>/meta-data/iam/info', strict_slashes=False)
@app.route('/<api_version>/meta-data/iam/info/<path:junk>')
def get_iam_info(api_version, junk=None):
    role_params_from_ip = roles.get_role_params_from_ip(request.remote_addr)
    if role_params_from_ip['name']:
        log.debug('Providing IAM role info for {0}'.format(role_params_from_ip['name']))
        return jsonify(roles.get_role_info_from_params(role_params_from_ip))
    else:
        log.error('Role name not found; returning 404.')
        return '', 404


@app.route('/<api_version>/meta-data/iam/security-credentials')
def get_security_credentials_noslash(api_version):
    return redirect(
        url_for('get_security_credentials_slash', api_version=api_version),
        code=301
    )


@app.route('/<api_version>/meta-data/iam/security-credentials/')
def get_security_credentials_slash(api_version):
    role_params = roles.get_role_params_from_ip(request.remote_addr)
    if not role_params['name']:
        return '', 404
    return role_params['name'], 200


@app.route(
    '/<api_version>/meta-data/iam/security-credentials/<requested_role>',
    methods=['GET'],
    strict_slashes=False
)
@app.route(
    '/<api_version>/meta-data/iam/security-credentials/<requested_role>/<path:junk>',
    methods=['GET']
)
def get_role_credentials(api_version, requested_role, junk=None):
    try:
        role_params = roles.get_role_params_from_ip(
            request.remote_addr,
            requested_role=requested_role
        )
    except roles.UnexpectedRoleError:
        return '', 403

    try:
        assumed_role = roles.get_assumed_role_credentials(
            role_params=role_params,
            api_version=api_version
        )
    except roles.GetRoleError as e:
        return '', e.args[0][0]
    return jsonify(assumed_role)


@app.route('/<api_version>/meta-data/instance-action')
def get_instance_action(api_version):
    return 'none', 200


@app.route('/<api_version>/meta-data/instance-id')
def get_instance_id(api_version):
    return 'i-{0}'.format(app.config['MOCKED_INSTANCE_ID']), 200


@app.route('/<api_version>/meta-data/instance-type')
def get_instance_type(api_version):
    return 't2.medium', 200


@app.route('/<api_version>/meta-data/mac')
def get_mac(api_version):
    return 'AE-30-76-CE-38-62', 200


@app.route('/<api_version>/meta-data/metrics')
def get_metrics_noslash(api_version):
    return redirect(
        url_for('get_metrics_slash', api_version=api_version),
        code=301
    )


@app.route('/<api_version>/meta-data/metrics/')
def get_metrics_slash(api_version):
    return 'vhostmd', 200


@app.route('/<api_version>/meta-data/metrics/vhostmd')
def get_metrics_vhostmd(api_version):
    return '<?xml version="1.0" encoding="UTF-8"?>', 200


@app.route('/<api_version>/meta-data/network')
def get_network_noslash(api_version):
    return redirect(
        url_for('get_network_slash', api_version=api_version),
        code=301
    )


@app.route('/<api_version>/meta-data/network/')
def get_network_slash(api_version):
    return 'interfaces/', 200


@app.route('/<api_version>/meta-data/network/interfaces')
def get_network_interfaces_noslash(api_version):
    return redirect(
        url_for('get_network_interfaces_slash', api_version=api_version),
        code=301
    )


@app.route('/<api_version>/meta-data/network/interfaces/')
def get_network_interfaces_slash(api_version):
    return 'macs/', 200


@app.route('/<api_version>/meta-data/network/interfaces/macs')
def get_network_interfaces_macs_noslash(api_version):
    return redirect(
        url_for('get_network_interfaces_macs_slash', api_version=api_version),
        code=301
    )


@app.route('/<api_version>/meta-data/network/interfaces/macs/')
def get_network_interfaces_macs_slash(api_version):
    return 'AE:30:76:CE:38:62/', 200


@app.route('/<api_version>/meta-data/network/interfaces/macs/AE:30:76:CE:38:62')
def get_network_interfaces_macaddr_noslash(api_version):
    return redirect(
        url_for(
            'get_network_interfaces_macaddr_slash',
            api_version=api_version
        ),
        code=301
    )


@app.route('/<api_version>/meta-data/network/interfaces/macs/AE:30:76:CE:38:62/')
def get_network_interfaces_macaddr_slash(api_version):
    info = [
        'device-number', 'interface-id', 'ipv4-associations/',
        'local-hostname', 'local-ipv4s', 'mac', 'owner-id', 'public-hostname',
        'public-ipv4s', 'security-group-ids', 'security-groups', 'subnet-id',
        'subnet-ipv4-cidr-block', 'vpc-id', 'vpc-ipv4-cidr-block'
    ]
    return '\n'.join(info), 200


@app.route('/<api_version>/meta-data/network/interfaces/macs/AE:30:76:CE:38:62/device-number')
def get_macaddr_device_number(api_version):
    return '0', 200


@app.route('/<api_version>/meta-data/network/interfaces/macs/AE:30:76:CE:38:62/interface-id')
def get_macaddr_interface_id(api_version):
    return 'eni-1234', 200


@app.route('/<api_version>/meta-data/network/interfaces/macs/AE:30:76:CE:38:62/ipv4-associations')
def get_macaddr_ipv4_associations_noslash(api_version):
    return redirect(
        url_for(
            'get_macaddr_ipv4_associations_slash',
            api_version=api_version
        ),
        code=301
    )


@app.route('/<api_version>/meta-data/network/interfaces/macs/AE:30:76:CE:38:62/ipv4-associations/')
def get_macaddr_ipv4_associations_slash(api_version):
    return '127.255.0.1', 200


@app.route('/<api_version>/meta-data/network/interfaces/macs/AE:30:76:CE:38:62/local-hostname')
def get_macaddr_local_hostname(api_version):
    return 'mocked.internal', 200


@app.route('/<api_version>/meta-data/network/interfaces/macs/AE:30:76:CE:38:62/local-ipv4s')
def get_macaddr_local_ipv4s(api_version):
    return 'mocked.internal', 200


@app.route('/<api_version>/meta-data/network/interfaces/macs/AE:30:76:CE:38:62/mac')
def get_macaddr_mac(api_version):
    return 'AE:30:76:CE:38:62', 200


@app.route('/<api_version>/meta-data/network/interfaces/macs/AE:30:76:CE:38:62/owner-id')
def get_macaddr_owner_id(api_version):
    return '12345', 200


@app.route('/<api_version>/meta-data/network/interfaces/macs/AE:30:76:CE:38:62/public-hostname')
def get_macaddr_public_hostname(api_version):
    return 'mocked.internal', 200


@app.route('/<api_version>/meta-data/network/interfaces/macs/AE:30:76:CE:38:62/public-ipv4s')
def get_macaddr_public_ipv4s(api_version):
    return '127.255.0.1', 200


@app.route('/<api_version>/meta-data/network/interfaces/macs/AE:30:76:CE:38:62/security-group-ids')
def get_macaddr_security_group_ids(api_version):
    return 'sg-1234', 200


@app.route('/<api_version>/meta-data/network/interfaces/macs/AE:30:76:CE:38:62/security-groups')
def get_macaddr_security_groups(api_version):
    return 'default', 200


@app.route('/<api_version>/meta-data/network/interfaces/macs/AE:30:76:CE:38:62/subnet-id')
def get_macaddr_subnet_id(api_version):
    return 'subnet-1234', 200


@app.route('/<api_version>/meta-data/network/interfaces/macs/AE:30:76:CE:38:62/subnet-ipv4-cidr-block')
def get_macaddr_subnet_ipv4_cidr_block(api_version):
    return '127.255.0.0/20', 200


@app.route('/<api_version>/meta-data/network/interfaces/macs/AE:30:76:CE:38:62/vpc-id')
def get_macaddr_vpc_id(api_version):
    return 'vpc-1234', 200


@app.route('/<api_version>/meta-data/network/interfaces/macs/AE:30:76:CE:38:62/vpc-ipv4-cidr-block')
def get_macaddr_vpc_ipv4_cidr_block(api_version):
    return '127.255.0.0/16', 200


@app.route('/<api_version>/meta-data/placement')
def get_placement_noslash(api_version):
    return redirect(
        url_for('get_placement_slash', api_version=api_version),
        code=301
    )


@app.route('/<api_version>/meta-data/placement/')
def get_placement_slash(api_version):
    return 'availability-zone', 200


@app.route('/<api_version>/meta-data/placement/availability-zone')
def get_placement_az(api_version):
    return 'us-east-1a', 200


@app.route('/<api_version>/meta-data/profile')
def get_profile(api_version):
    return 'default-hvm', 200


@app.route('/<api_version>/meta-data/public-hostname')
def get_public_hostname(api_version):
    return 'mocked.internal', 200


@app.route('/<api_version>/meta-data/public-ipv4')
def get_public_ipv4s(api_version):
    return '127.255.0.1', 200


@app.route('/<api_version>/meta-data/public-keys')
def get_public_keys_noslash(api_version):
    return redirect(
        url_for('get_public_keys_slash', api_version=api_version),
        code=301
    )


@app.route('/<api_version>/meta-data/public-keys/')
def get_public_keys_slash(api_version):
    return '0=boot', 200


@app.route('/<api_version>/meta-data/reservation-id')
def get_reservation_id(api_version):
    return 'r-1234', 200


@app.route('/<api_version>/meta-data/security-groups')
def get_security_groups(api_version):
    return 'default', 200


@app.route('/<api_version>/meta-data/services')
def get_services_noslash(api_version):
    return redirect(
        url_for('get_services_slash', api_version=api_version),
        code=301
    )


@app.route('/<api_version>/meta-data/services/')
def get_services_slash(api_version):
    return 'domain', 200


@app.route('/<api_version>/meta-data/services/domain')
def get_services_domain(api_version):
    return 'amazonaws.com', 200


@app.route('/<api_version>/dynamic')
def get_dynamic_noslash(api_version):
    return redirect(
        url_for('get_dynamic_slash', api_version=api_version),
        code=301
    )


@app.route('/<api_version>/dynamic/')
def get_dynamic_slash(api_version):
    return 'instance-identity/\nfws/\n', 200


@app.route('/<api_version>/dynamic/instance-identity')
def get_instance_identity_noslash(api_version):
    return redirect(
        url_for('get_instance_identity_slash', api_version=api_version),
        code=301
    )


@app.route('/<api_version>/dynamic/instance-identity/')
def get_instance_identity_slash(api_version):
    return 'document\npkcs7\nsignature\ndsa2048', 200


@app.route('/<api_version>/dynamic/instance-identity/document')
def get_instance_identity_document(api_version):
    time_format = "%Y-%m-%dT%H:%M:%SZ"
    now = datetime.datetime.now(dateutil.tz.tzutc())
    ret = {
        'privateIp': '127.255.0.1',
        'devpayProductCodes': None,
        'availabilityZone': 'us-east-1a',
        'version': '2010-08-31',
        'accountId': '1234',
        'instanceId': 'i-{0}'.format(app.config['MOCKED_INSTANCE_ID']),
        'billingProducts': None,
        'instanceType': 't2.medium',
        # This may be a terrible mock for this...
        'pendingTime': now.strftime(time_format),
        'imageId': 'ami-1234',
        'kernelId': None,
        'ramdiskId': None,
        'architecture': 'x86_64',
        'region': 'us-east-1'
    }
    return jsonify(ret)


@app.route('/<api_version>/dynamic/instance-identity/pkcs7')
def get_instance_identity_pkcs7(api_version):
    # TODO: determine a reasonable mock for this
    return 'mocked', 200


@app.route('/<api_version>/dynamic/instance-identity/signature')
def get_instance_identity_signature(api_version):
    # TODO: determine a reasonable mock for this
    return 'mocked', 200


@app.route('/<api_version>/dynamic/instance-identity/dsa2048')
def get_instance_identity_dsa2048(api_version):
    # TODO: determine a reasonable mock for this
    return 'mocked', 200


@app.route('/<api_version>/dynamic/fws')
def get_fws_noslash(api_version):
    return redirect(
        url_for('get_fws_slash', api_version=api_version),
        code=301
    )


@app.route('/<api_version>/dynamic/fws/')
def get_fws_slash(api_version):
    return 'instance-monitoring\n', 200


@app.route('/<api_version>/dynamic/fws/instance-monitoring')
def get_instance_monitoring(api_version):
    return 'enabled', 200



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import setup, find_packages

with open('requirements.txt') as f:
    reqs_base = f.read().splitlines()
    reqs_base = [r for r in reqs_base if not r.startswith('#') and r]

with open('requirements_wsgi.txt') as f:
    reqs_wsgi = f.read().splitlines()
    reqs_wsgi = [r for r in reqs_wsgi if not r.startswith('#') and r]

reqs = reqs_base + reqs_wsgi

setup(
    name="metadataproxy",
    version="1.10.0",
    packages=find_packages(exclude=["test*"]),
    include_package_data=True,
    zip_safe=False,
    install_requires=reqs,
    author="Ryan Lane",
    author_email="rlane@lyft.com",
    description=("A proxy for AWS's metadata service that gives out"
                 " scoped IAM credentials from STS"),
    license="apache2",
    url="https://github.com/lyft/metadataproxy"
)



import logging

from flask import Flask

from metadataproxy import settings

log = logging.getLogger(__name__)

app = Flask(__name__)
app.config.from_object(settings)
app.debug = app.config['DEBUG']

if app.config['MOCK_API']:
    from metadataproxy.routes import mock  # NOQA
else:
    from metadataproxy.routes import proxy  # NOQA
